@@ -1,19 +1,7 @@
-from typing import Sequence, Tuple, Union
-
+from typing import Optional, Sequence, Tuple, Union
 import numpy
-from dask.array.core import Array
-from numpy import (
-    array,
-    array2string,
-    dtype,
-    get_printoptions,
-    integer,
-    issubdtype,
-    ndarray,
-    prod,
-)
+from numpy import array, dtype, integer, issubdtype, ndarray, prod, array2string
 
-import warnings
 from .BinaryIsometricOp import BinaryIsometricOp
 from .Cast import Cast
 from .Combine import Combine
@@ -22,7 +10,14 @@ from .Subset import Subset
 from .Transpose import Transpose
 from .UnaryIsometricOpSimple import UnaryIsometricOpSimple
 from .UnaryIsometricOpWithArgs import UnaryIsometricOpWithArgs
-from .utils import create_dask_array, extract_array, _densify
+
+from ._subset import _getitem_subset_preserves_dimensions, _getitem_subset_discards_dimensions, _repr_subset
+from ._isometric import translate_ufunc_to_op_simple, translate_ufunc_to_op_with_args
+from .extract_dense_array import extract_dense_array
+from .extract_sparse_array import extract_sparse_array
+from .create_dask_array import create_dask_array
+from .chunk_shape import chunk_shape
+from .is_sparse import is_sparse
 
 __author__ = "ltla"
 __copyright__ = "ltla"
@@ -58,98 +53,69 @@ def _extract_seed(x):
         return x
 
 
-translate_ufunc_to_op_with_args = set(
-    [
-        "add",
-        "subtract",
-        "multiply",
-        "divide",
-        "remainder",
-        "floor_divide",
-        "power",
-        "equal",
-        "greater_equal",
-        "greater",
-        "less_equal",
-        "less",
-        "not_equal",
-        "logical_and",
-        "logical_or",
-        "logical_xor",
-    ]
-)
-
-translate_ufunc_to_op_simple = set(
-    [
-        "log",
-        "log1p",
-        "log2",
-        "log10",
-        "exp",
-        "expm1",
-        "sqrt",
-        "sin",
-        "cos",
-        "tan",
-        "sinh",
-        "cosh",
-        "tanh",
-        "arcsin",
-        "arccos",
-        "arctan",
-        "arcsinh",
-        "arccosh",
-        "arctanh",
-        "ceil",
-        "floor",
-        "trunc",
-        "sign",
-    ]
-)
-
-
 class DelayedArray:
-    """Array containing delayed operations.
+    """Array containing delayed operations. This is equivalent to the class of
+    the same name from the `R/Bioconductor package
+    <https://bioconductor.org/packages/DelayedArray>`_ of the same name.  It
+    allows users to efficiently operate on large matrices without actually
+    evaluating the operation or creating new copies; instead, the operations
+    will transparently return another ``DelayedArray`` instance containing the
+    delayed operations, which can be realized by calling
+    :py:meth:`~numpy.array` or related methods.
 
-    This is equivalent to the class of the same name from
-    the `R/Bioconductor package <https://bioconductor.org/packages/DelayedArray>`_ of the same name.
-    It allows users to efficiently operate on large matrices without actually evaluating the
-    operation or creating new copies; instead, the operations will transparently return another ``DelayedArray``
-    instance containing the delayed operations, which can be realized by calling :py:meth:`~numpy.array` or related
-    methods.
+    Any object that satisfies the "seed contract" can be wrapped by a
+    ``DelayedArray``. Specifically, a seed should have:
 
-    Attributes:
-        seed: Any array-like object that satisfies the seed contract.
-            This means that it has the :py:attr:`~shape` and :py:attr:`~dtype` properties.
-            It should also NumPy slicing via :py:meth:`~numpy.ix_`; additional NumPy
-            interface support will be used where relevant.
+    - The :py:attr:`~shape` and :py:attr:`~dtype` properties, which are of the
+      same type as the corresponding properties of NumPy arrays.
+    - A method for the
+      :py:meth:`~delayedarray.extract_dense_array.extract_dense_array` generic.
+
+    If the seed contains sparse data, it should also implement:
+
+    - A method for the :py:meth:`~delayedarray.is_sparse.is_sparse` generic.
+    - A method for the
+      :py:meth:`~delayedarray.extract_sparse_array.extract_sparse_array`
+      generic.
+
+    Optionally, a seed class may have:
+
+    - A method for the :py:meth:`~delayedarray.chunk_shape.chunk_shape` generic,
+      if there is some preferred dimension in which to take chunks of the array.
+    - A method for the
+      :py:meth:`~delayedarray.create_dask_array.create_dask_array` generic,
+      if the seed is not already compatible with the **dask** package.
     """
 
     def __init__(self, seed):
+        """Most users should use :py:meth:`~delayedarray.wrap.wrap`
+        instead, as this can be specialized by developers to construct
+        subclasses that are optimized for custom seed types.
+
+        Args:
+            seed: Any array-like object that satisfies the seed contract.
+        """
         self._seed = seed
 
     @property
     def shape(self) -> Tuple[int, ...]:
-        """Shape of the ``DelayedArray``.
-
+        """
         Returns:
-            Tuple[int, ...]: Tuple of integers specifying the extent of each dimension of the ``DelayedArray``.
+            Tuple of integers specifying the extent of each dimension of the ``DelayedArray``.
         """
         return self._seed.shape
 
     @property
     def dtype(self) -> dtype:
-        """Type of the elements in the ``DelayedArray``.
-
+        """
         Returns:
-            dtype: NumPy type of the values.
+            NumPy type of the elements in the ``DelayedArray``.
         """
         return self._seed.dtype
 
     @property
     def seed(self):
-        """Get the underlying object satisfying the seed contract.
-
+        """
         Returns:
             The seed object.
         """
@@ -157,68 +123,53 @@ class DelayedArray:
 
     @property
     def T(self) -> "DelayedArray":
-        """Get the delayed transpose of this ``DelayedArray`` instance.
-
+        """
         Returns:
-            DelayedArray: A DelayedArray containing a delayed transposition.
+            A ``DelayedArray`` containing the delayed transpose.
         """
         return DelayedArray(Transpose(self._seed, perm=None))
 
     def __repr__(self) -> str:
-        """Pretty-print this ``DelayedArray``. This uses :py:meth:`~numpy.array2string` and responds to all of its
-        options.
+        """Pretty-print this ``DelayedArray``. This uses
+        :py:meth:`~numpy.array2string` and responds to all of its options.
 
         Returns:
-            str: String containing a prettified display of the array contents.
+            String containing a prettified display of the array contents.
         """
-        total = 1
-        for s in self._seed.shape:
-            total *= s
-
         preamble = "<" + " x ".join([str(x) for x in self._seed.shape]) + ">"
-        preamble += " DelayedArray object of type '" + self._seed.dtype.name + "'"
+        if is_sparse(self._seed):
+            preamble += " sparse"
+        preamble += " " + type(self).__name__ + " object of type '" + self._seed.dtype.name + "'"
 
-        indices = None
-        if total > get_printoptions()["threshold"]:
-            ndims = len(self._seed.shape)
-            indices = []
-            edge_size = get_printoptions()["edgeitems"]
-            for d in range(ndims):
-                extent = self._seed.shape[d]
-                if extent > edge_size * 2:
-                    indices.append(
-                        list(range(edge_size + 1))
-                        + list(range(extent - edge_size, extent))
-                    )
-                else:
-                    indices.append(slice(None))
-            indices = (*indices,)
-
-        bits_and_pieces = _densify(extract_array(self._seed, indices))
+        indices = _repr_subset(self._seed.shape)
+        bits_and_pieces = extract_dense_array(self._seed, indices)
         converted = array2string(bits_and_pieces, separator=", ", threshold=0)
         return preamble + "\n" + converted
 
     # For NumPy:
     def __array__(self) -> ndarray:
-        """Convert a ``DelayedArray`` to a NumPy array.
+        """Convert a ``DelayedArray`` to a NumPy array, to be used by
+        :py:meth:`~numpy.array`. 
 
         Returns:
-            ndarray: Array of the same type as :py:attr:`~dtype` and shape as :py:attr:`~shape`.
-            This is guaranteed to be in C-contiguous order and to not be a view on other data.
+            NumPy array of the same type as :py:attr:`~dtype` and shape as
+            :py:attr:`~shape`.  This is guaranteed to be in Fortran storage
+            order and to not be a view on other data.
         """
-        return _densify(extract_array(self._seed))
+        return extract_dense_array(self._seed)
 
     def __array_ufunc__(self, ufunc, method, *inputs, **kwargs) -> "DelayedArray":
-        """Interface with NumPy array methods.
+        """Interface with NumPy array methods. This is used to implement
+        mathematical operations like NumPy's :py:meth:`~numpy.log`, or to
+        override operations between NumPy class instances and ``DelayedArray``
+        objects where the former is on the left hand side. 
 
-        This is used to implement mathematical operations like NumPy's :py:meth:`~numpy.log`,
-        or to override operations between NumPy class instances and ``DelayedArray`` objects where the former is on the
-        left hand side. Check out the NumPy's ``__array_ufunc__``
-        `documentation <https://numpy.org/doc/stable/reference/arrays.classes.html#numpy.class.__array_ufunc__>`_ for
-        more details.
+        Check out NumPy's ``__array_ufunc__`` `documentation
+        <https://numpy.org/doc/stable/reference/arrays.classes.html#numpy.class.__array_ufunc__>`_
+        for more details.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` instance containing the requested delayed operation.
+            A ``DelayedArray`` instance containing the requested delayed operation.
         """
         if (
             ufunc.__name__ in translate_ufunc_to_op_with_args
@@ -250,15 +201,16 @@ class DelayedArray:
 
         raise NotImplementedError(f"'{ufunc.__name__}' is not implemented!")
 
-    def __array_function__(self, func, types, args, kwargs):
-        """Interface to NumPy's high-level array functions.
-        This is used to implement array operations like NumPy's :py:meth:`~numpy.concatenate`,
-        Check out the NumPy's ``__array_function__``
-        `documentation <https://numpy.org/doc/stable/reference/arrays.classes.html#numpy.class.__array_function__>`_
+    def __array_function__(self, func, types, args, kwargs) -> "DelayedArray":
+        """Interface to NumPy's high-level array functions.  This is used to
+        implement array operations like NumPy's :py:meth:`~numpy.concatenate`,
+
+        Check out NumPy's ``__array_function__`` `documentation
+        <https://numpy.org/doc/stable/reference/arrays.classes.html#numpy.class.__array_function__>`_
         for more details.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` instance containing the requested delayed operation.
+            A ``DelayedArray`` instance containing the requested delayed operation.
         """
         if func == numpy.concatenate:
             seeds = []
@@ -305,10 +257,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed addition operation.
+            A ``DelayedArray`` containing the delayed addition operation.
         """
         return _wrap_isometric_with_args(self, other, operation="add", right=True)
 
@@ -320,10 +272,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed addition operation.
+            A ``DelayedArray`` containing the delayed addition operation.
         """
         return _wrap_isometric_with_args(self, other, operation="add", right=False)
 
@@ -335,10 +287,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed subtraction operation.
+            A ``DelayedArray`` containing the delayed subtraction operation.
         """
         return _wrap_isometric_with_args(self, other, operation="subtract", right=True)
 
@@ -350,10 +302,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed subtraction operation.
+            A ``DelayedArray`` containing the delayed subtraction operation.
         """
         return _wrap_isometric_with_args(self, other, operation="subtract", right=False)
 
@@ -365,10 +317,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed multiplication operation.
+            A ``DelayedArray`` containing the delayed multiplication operation.
         """
         return _wrap_isometric_with_args(self, other, operation="multiply", right=True)
 
@@ -380,10 +332,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed multiplication operation.
+            A ``DelayedArray`` containing the delayed multiplication operation.
         """
         return _wrap_isometric_with_args(self, other, operation="multiply", right=False)
 
@@ -395,10 +347,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed division operation.
+            A ``DelayedArray`` containing the delayed division operation.
         """
         return _wrap_isometric_with_args(self, other, operation="divide", right=True)
 
@@ -410,10 +362,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed division operation.
+            A ``DelayedArray`` containing the delayed division operation.
         """
         return _wrap_isometric_with_args(self, other, operation="divide", right=False)
 
@@ -425,10 +377,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` object of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed modulo operation.
+            A ``DelayedArray`` containing the delayed modulo operation.
         """
         return _wrap_isometric_with_args(self, other, operation="remainder", right=True)
 
@@ -440,10 +392,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed modulo operation.
+            A ``DelayedArray`` containing the delayed modulo operation.
         """
         return _wrap_isometric_with_args(
             self, other, operation="remainder", right=False
@@ -457,10 +409,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed floor division operation.
+            A ``DelayedArray`` containing the delayed floor division operation.
         """
         return _wrap_isometric_with_args(
             self, other, operation="floor_divide", right=True
@@ -474,10 +426,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed floor division operation.
+            A ``DelayedArray`` containing the delayed floor division operation.
         """
         return _wrap_isometric_with_args(
             self, other, operation="floor_divide", right=False
@@ -491,10 +443,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed power operation.
+            A ``DelayedArray`` containing the delayed power operation.
         """
         return _wrap_isometric_with_args(self, other, operation="power", right=True)
 
@@ -506,10 +458,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed power operation.
+            A ``DelayedArray`` containing the delayed power operation.
         """
         return _wrap_isometric_with_args(self, other, operation="power", right=False)
 
@@ -521,10 +473,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed check.
+            A ``DelayedArray`` containing the delayed check.
         """
         return _wrap_isometric_with_args(self, other, operation="equal", right=True)
 
@@ -536,10 +488,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed check.
+            A ``DelayedArray`` containing the delayed check.
         """
         return _wrap_isometric_with_args(self, other, operation="equal", right=False)
 
@@ -551,10 +503,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed check.
+            A ``DelayedArray`` containing the delayed check.
         """
         return _wrap_isometric_with_args(self, other, operation="not_equal", right=True)
 
@@ -566,10 +518,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed check.
+            A ``DelayedArray`` containing the delayed check.
         """
         return _wrap_isometric_with_args(
             self, other, operation="not_equal", right=False
@@ -583,10 +535,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed check.
+            A ``DelayedArray`` containing the delayed check.
         """
         return _wrap_isometric_with_args(
             self, other, operation="greater_equal", right=True
@@ -600,10 +552,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed check.
+            A ``DelayedArray`` containing the delayed check.
         """
         return _wrap_isometric_with_args(
             self, other, operation="greater_equal", right=False
@@ -617,10 +569,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed check.
+            A ``DelayedArray`` containing the delayed check.
         """
         return _wrap_isometric_with_args(
             self, other, operation="less_equal", right=True
@@ -634,10 +586,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed check.
+            A ``DelayedArray`` containing the delayed check.
         """
         return _wrap_isometric_with_args(
             self, other, operation="less_equal", right=False
@@ -651,10 +603,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed check.
+            A ``DelayedArray`` containing the delayed check.
         """
         return _wrap_isometric_with_args(self, other, operation="greater", right=True)
 
@@ -666,10 +618,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed check.
+            A ``DelayedArray`` containing the delayed check.
         """
         return _wrap_isometric_with_args(self, other, operation="greater", right=False)
 
@@ -681,10 +633,10 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed check.
+            A ``DelayedArray`` containing the delayed check.
         """
         return _wrap_isometric_with_args(self, other, operation="less", right=True)
 
@@ -696,195 +648,97 @@ class DelayedArray:
                 A numeric scalar;
                 or a NumPy array with dimensions as described in
                 :py:class:`~delayedarray.UnaryIsometricOpWithArgs.UnaryIsometricOpWithArgs`;
-                or any seed object of the same dimensions as :py:attr:`~shape`.
+                or a ``DelayedArray`` of the same dimensions as :py:attr:`~shape`.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed check.
+            A ``DelayedArray`` containing the delayed check.
         """
         return _wrap_isometric_with_args(self, other, operation="less", right=False)
 
     # Simple methods.
-    def __neg__(self):
+    def __neg__(self) -> "DelayedArray":
         """Negate the contents of a ``DelayedArray``.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed negation.
+            A ``DelayedArray`` containing the delayed negation.
         """
         return _wrap_isometric_with_args(self, 0, operation="subtract", right=False)
 
-    def __abs__(self):
+    def __abs__(self) -> "DelayedArray":
         """Take the absolute value of the contents of a ``DelayedArray``.
 
         Returns:
-            DelayedArray: A ``DelayedArray`` containing the delayed absolute value operation.
+            A ``DelayedArray`` containing the delayed absolute value operation.
         """
         return DelayedArray(UnaryIsometricOpSimple(self._seed, operation="abs"))
 
     # Subsetting.
-    def __getitem__(
-        self, args: Tuple[Union[slice, Sequence[Union[int, bool]]], ...]
-    ) -> Union["DelayedArray", ndarray]:
+    def __getitem__(self, subset: Tuple[Union[slice, Sequence], ...]) -> Union["DelayedArray", ndarray]:
         """Take a subset of this ``DelayedArray``. This follows the same logic as NumPy slicing and will generate a
         :py:class:`~delayedarray.Subset.Subset` object when the subset operation preserves the dimensionality of the
         seed, i.e., ``args`` is defined using the :py:meth:`~numpy.ix_` function.
 
         Args:
-            args (Tuple[Union[slice, Sequence[Union[int, bool]]], ...]):
-                A :py:class:`tuple` of length equal to the dimensionality of this ``DelayedArray``.
-                Any NumPy slicing is supported but only subsets that preserve dimensionality will generate a
-                delayed subset operation.
-
-        Raises:
-            ValueError: If ``args`` contain more dimensions than the shape of the array.
+            subset:
+                A :py:class:`tuple` of length equal to the dimensionality of
+                this ``DelayedArray``.  We attempt to support most types of
+                NumPy slicing; however, only subsets that preserve
+                dimensionality will generate a delayed subset operation.
 
         Returns:
-            If the dimensionality is preserved by ``args``, a ``DelayedArray`` containing a delayed subset operation is
-            returned. Otherwise, a :py:class:`~numpy.ndarray` is returned containing the realized subset.
+            If the dimensionality is preserved by ``subset``, a
+            ``DelayedArray`` containing a delayed subset operation is returned.
+            Otherwise, a :py:class:`~numpy.ndarray` is returned containing the
+            realized subset.
         """
+        cleaned = _getitem_subset_preserves_dimensions(self.shape, subset)
+        if cleaned is not None:
+            return DelayedArray(Subset(self._seed, cleaned))
+        return _getitem_subset_discards_dimensions(self._seed, subset, extract_dense_array)
 
-        ndim = len(self.shape)
-        if not isinstance(args, tuple):
-            args = [args] + [slice(None)] * (ndim - 1)
-        if len(args) < ndim:
-            args = list(args) + [slice(None)] * (ndim - len(args))
-        elif len(args) > ndim:
-            raise ValueError(
-                "more indices in 'args' than there are dimensions in 'seed'"
-            )
-
-        # Checking if we're preserving the shape via a cross index.
-        cross_index = True
-        for d, idx in enumerate(args):
-            if (
-                not isinstance(idx, ndarray)
-                or not issubdtype(idx.dtype, integer)
-                or len(idx.shape) != ndim
-            ):
-                cross_index = False
-                break
-
-            for d2 in range(ndim):
-                if d != d2 and idx.shape[d2] != 1:
-                    cross_index = False
-                    break
-
-        if cross_index:
-            sanitized = []
-            for d, idx in enumerate(args):
-                sanitized.append(idx.reshape((prod(idx.shape),)))
-            return DelayedArray(Subset(self._seed, (*sanitized,)))
-
-        # Checking if we're preserving the shape via a slice.
-        slices = 0
-        failed = False
-        for d, idx in enumerate(args):
-            if isinstance(idx, slice):
-                slices += 1
-                continue
-            elif isinstance(idx, ndarray):
-                if len(idx.shape) != 1:
-                    failed = True
-                    break
-            elif not isinstance(idx, Sequence):
-                failed = True
-                break
-
-        if not failed and slices >= ndim - 1:
-            sanitized = []
-            for d, idx in enumerate(args):
-                if isinstance(idx, slice):
-                    sanitized.append(range(*idx.indices(self.shape[d])))
-                else:
-                    dummy = array(range(self.shape[d]))[idx]
-                    sanitized.append(dummy)
-            return DelayedArray(Subset(self._seed, (*sanitized,)))
-
-        # If we're discarding dimensions, we see if we can do some pre-emptive extraction.
-        failed = False
-        as_vector = []
-        new_args = []
-        dim_loss = 0
-
-        for d, idx in enumerate(args):
-            if isinstance(idx, ndarray):
-                if len(idx.shape) != 1:
-                    failed = True
-                    break
-            elif isinstance(idx, slice):
-                idx = range(*idx.indices(self.shape[d]))
-            elif not isinstance(idx, Sequence):
-                as_vector.append([idx])
-                new_args.append(0)
-                dim_loss += 1
-                continue
-
-            as_vector.append(idx)
-            new_args.append(slice(None))
-
-        if not failed:
-            # Just using Subset here to avoid having to reproduce the
-            # uniquifying/sorting of subsets before extract_array().
-            base_seed = extract_array(Subset(self._seed, (*as_vector,)))
-        else:
-            base_seed = extract_array(self._seed)
-            new_args = args
-
-        try:
-            test = base_seed[(..., *new_args)]
-            if len(test.shape) != ndim - dim_loss:
-                raise ValueError(
-                    "slicing for "
-                    + str(type(base_seed))
-                    + " does not discard dimensions with scalar indices"
-                )
-        except Exception as e:
-            warnings.warn(e)
-            test = _densify(base_seed)[(..., *new_args)]
-
-        if len(test.shape) == ndim:
-            raise NotImplementedError(
-                "Oops. Looks like the DelayedArray doesn't correctly handle this combination of index types, but it "
-                "probably should. Consider filing an issue in at https://github.com/BiocPy/DelayedArray/issues."
-            )
-
-        return test
 
     # For python-level compute.
     def sum(self, *args, **kwargs):
         """See :py:meth:`~numpy.sums` for details."""
-        target = extract_array(self._seed)
-        try:
-            return target.sum(*args, **kwargs)
-        except Exception as e:
-            warnings.warn(e)
-            target = _densify(target)
-            return target.sum(*args, **kwargs)
+        target = extract_dense_array(self._seed)
+        return target.sum(*args, **kwargs)
 
     def var(self, *args, **kwargs):
         """See :py:meth:`~numpy.vars` for details."""
-        target = extract_array(self._seed)
-        try:
-            return target.var(*args, **kwargs)
-        except Exception as e:
-            warnings.warn(e)
-            target = _densify(target)
-            return target.var(*args, **kwargs)
+        target = extract_dense_array(self._seed)
+        return target.var(*args, **kwargs)
 
     def mean(self, *args, **kwargs):
         """See :py:meth:`~numpy.means` for details."""
-        target = extract_array(self._seed)
-        try:
-            return target.mean(*args, **kwargs)
-        except Exception as e:
-            warnings.warn(e)
-            target = _densify(target)
-            return target.mean(*args, **kwargs)
+        target = extract_dense_array(self._seed)
+        return target.mean(*args, **kwargs)
 
-    # Coercion methods.
-    def __DelayedArray_dask__(self) -> Array:
-        """See :py:meth:`~delayedarray.utils.create_dask_array`."""
-        return create_dask_array(self._seed)
 
-    def __DelayedArray_extract__(self, subset: Tuple[Sequence[int]]) -> Array:
-        """See :py:meth:`~delayedarray.utils.extract_array`."""
-        return extract_array(self._seed, subset)
+@extract_dense_array.register
+def extract_dense_array_DelayedArray(x: DelayedArray, subset: Optional[Tuple[Sequence[int], ...]] = None):
+    """See :py:meth:`~delayedarray.extract_dense_array.extract_dense_array`."""
+    return extract_dense_array(x._seed, subset)
+
+
+@extract_sparse_array.register
+def extract_sparse_array_DelayedArray(x: DelayedArray, subset: Optional[Tuple[Sequence[int], ...]] = None):
+    """See :py:meth:`~delayedarray.extract_sparse_array.extract_sparse_array`."""
+    return extract_sparse_array(x._seed, subset)
+
+
+@create_dask_array.register
+def create_dask_array_DelayedArray(x: DelayedArray):
+    """See :py:meth:`~delayedarray.create_dask_array.create_dask_array`."""
+    return create_dask_array(x._seed)
+
+
+@chunk_shape.register
+def chunk_shape_DelayedArray(x: DelayedArray):
+    """See :py:meth:`~delayedarray.chunk_shape.chunk_shape`."""
+    return chunk_shape(x._seed)
+
+
+@is_sparse.register
+def is_sparse_DelayedArray(x: DelayedArray):
+    """See :py:meth:`~delayedarray.is_sparse.is_sparse`."""
+    return is_sparse(x._seed)
